// rundir

// Copyright 2020 The Go Authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

// gccgo got an undefined symbol reference when inlining a method expression.
package ignored
